import hashlib
import os
import time

import boto3 as boto3
from werkzeug.utils import secure_filename

from gwlib.config import config
from gwlib.base.base_service import BaseService


class BaseUploadService(BaseService):

    def __init__(self):
        super().__init__()

    def upload(self, file, path=None):
        host = "https://{}.s3.amazonaws.com".format(config.BUCKET_NAME)
        filename = file.filename
        filename_splited = filename.split(".")
        if len(filename_splited) <= 1:
            raise Exception("MALFORMED filename")

        extension = filename_splited[-1]
        mimetype = file.content_type
        filename = hashlib.sha256(str(secure_filename(file.filename) +
                                      str(time.time())).encode('utf-8')).hexdigest() + ".{}".format(extension)
        s3 = boto3.client(
            "s3",
            aws_access_key_id=os.getenv('AWS_UPLOAD_KEY_ID'),
            aws_secret_access_key=os.getenv('AWS_UPLOAD_SECRET')
        )
        file_stream = file.stream.read()
        print("log", len(file_stream))
        filename = path.format(filename)
        s3.put_object(Key=filename, ACL='public-read',
                      ContentType=mimetype,
                      Bucket=config.BUCKET_NAME, Body=file_stream)
        file_url = "{}/{}".format(host, filename)
        return {'success': True,
                "file_url": file_url,
                "file_name": filename
                }





API_PATH = "/api/v1"


class Resource:
    GW_USER = "gw_user"
    GW_HOLE = "hole"
    GW_SENSOR = "sensor"
    GW_GOLF_COURSE = "golf_course"
    GW_ALERT = "alert"
    GW_MENU = "menu"
    GW_REPORT = "report"



from sqlalchemy import func
from sqlalchemy.orm import relationship

from gwlib.models import ModelJson
from . import db


class Hole(db.Model, ModelJson):
    __tablename__ = "gw_hole"

    hole_id = db.Column(db.Integer, primary_key=True)
    course_id = db.Column(db.Integer, db.ForeignKey('gw_golf_course.course_id'), nullable=False)
    zone = db.Column(db.String(length=20), nullable=True)
    name = db.Column(db.String(length=20), nullable=False)
    deleted = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)

    golf_course = relationship("GolfCourse")
    sensors = relationship("Sensor", lazy='dynamic')



from gwlib.dao import BaseUserDAO
from gwlib.base.base_service import BaseService
from gwlib.models import User
from gwlib.models import UserType


class BaseUserService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseUserDAO()

    def get_by_user_id(self, user_id):
        return self.dao.get(user_id=user_id)

    def get_by_username(self, username):
        return self.dao.get(username=username)

    def get_menu(self, user_id):
        user = self.dao.get(user_id=user_id)  # type: User
        user_type = user.user_type  # type: UserType
        return user_type.menus



from gwlib.base.base_service import BaseService
from gwlib.dao.holes_dao import BaseHolesDAO


class BaseHolesService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseHolesDAO()

    def get_by_name(self, resource_name):
        return self.dao.get(name=resource_name)



import os

from passlib.hash import pbkdf2_sha256
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail

class Helper:

    @staticmethod
    def verify_crypt(password, compare_password):
        return pbkdf2_sha256.verify(password, compare_password)

    @staticmethod
    def set_crypt(password):
        print("que pex", password)
        return pbkdf2_sha256.hash(password)


    @staticmethod
    def send_mail(to, subject, content):
        message = Mail(
            from_email='no-reply@groundworx.com',
            to_emails=to,
            subject=subject,
            html_content=content
        )
        try:
            sg = SendGridAPIClient(os.environ.get('SENDGRID_API_KEY', ""))
            response = sg.send(message)
            print(response.status_code)
            print(response.body)
            print(response.headers)
        except Exception as e:
            print("error", e)



from sqlalchemy import Table

from . import db

user_type_menu = Table('gw_user_type_menu',
                       db.Model.metadata,
                       db.Column('menu_id', db.Integer, db.ForeignKey('gw_menu.menu_id'), nullable=False),
                       db.Column('type_id', db.Integer, db.ForeignKey('gw_user_type.type_id'), nullable=False),
                       db.Column('deleted', db.Boolean, default=False, nullable=False)
                       )



from .golf_course_dao import BaseGolfCourseDAO
from .resource_dao import BaseResourceDAO
from .user_dao import BaseUserDAO
from .alert_dao import BaseAlertDAO
from .menu_dao import BaseMenuDAO
from .report_dao import BaseReportDAO


from gwlib.base.base_dao import BaseDAO
from gwlib.models import UserTypePolicy


class UserPolicyDAO(BaseDAO):
    model = UserTypePolicy

    def get_permissions_by_resource(self, type_id, resource_id):
        user_type_policy = self.get(**{"type_id": type_id, "resource_id": resource_id})  # type: UserTypePolicy
        return user_type_policy



from gwlib.base.base_service import BaseService
from gwlib.dao import BaseReportDAO


class BaseReportService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseReportDAO()



from flask_sqlalchemy import SQLAlchemy

db = SQLAlchemy()

from .Alert import *
from .GolfCourse import *
from .User import *
from .Hole import *
from .Sensor import *
from .Report import *
from .UserType import *
from .UserToken import *
from .UserTypePolicy import *
from .Resource import *
from .menu import *
from .ZoneLocation import *
from .ZoneQuadrant import *
from .Zone import *



from sqlalchemy import func
from sqlalchemy.orm import relationship

from gwlib.models import db
from .model_json import ModelJson


class UserTypePolicy(db.Model, ModelJson):
    __tablename__ = "gw_user_type_policy"

    policy_id = db.Column(db.Integer, primary_key=True)
    type_id = db.Column(db.Integer, db.ForeignKey('gw_user_type.type_id'), nullable=False)
    resource_id = db.Column(db.Integer, db.ForeignKey('gw_resource.resource_id'), nullable=False)
    can_create = db.Column(db.Boolean, default=False, nullable=False)
    can_read = db.Column(db.Boolean, default=False, nullable=False)
    can_update = db.Column(db.Boolean, default=False, nullable=False)
    can_delete = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)

    resource = relationship("Resource")



from sqlalchemy import func

from . import db


class AlertType(db.Model):
    __tablename__ = "gw_alert_type"

    type_id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(length=100), nullable=False)


class Alert(db.Model):
    __tablename__ = "gw_alert"

    course_id = db.Column(db.Integer, primary_key=True)
    sensor_id = db.Column(db.Integer, db.ForeignKey('gw_sensor.sensor_id'), nullable=False)
    type_id = db.Column(db.Integer, db.ForeignKey('gw_alert_type.type_id'), nullable=False)
    title = db.Column(db.String(length=100), nullable=False)
    text = db.Column(db.Text, nullable=False)
    threshold = db.Column(db.Text, nullable=False)
    deleted = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)



import datetime
import os

basedir = os.path.abspath(os.path.dirname(__file__))


class Config(object):
    # JWT CONF
    JWT_SECRET_KEY = os.environ.get('JWT_SECRET', "039urfjd9sf8usdf9ijdsf;lsdlf23/;el23p")
    SECRET_KEY = os.urandom(24)
    JWT_ACCESS_TOKEN_EXPIRES = datetime.timedelta(days=int(os.environ.get('JWT_EXPIRES', 7)))
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://{}:{}@{}/{}".format(
        os.environ.get('MYSQL_USER', "groundworx"),
        os.environ.get('MYSQL_PASS', "groundworx"),
        os.environ.get('MYSQL_HOST', "mysql"),
        os.environ.get('MYSQL_DB', "groundworx"),
    )
    JWT_ALGORITHM = os.environ.get('JWT_ALGORITHM', 'HS256')

    # DATABASE
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    APP_HOST = "http://test-bucket-gw.s3-website-us-west-1.amazonaws.com/recover-password"
    BUCKET_NAME = 'groundworx-media'

    @staticmethod
    def init_app(app):
        pass


from gwlib.base.base_service import BaseService
from gwlib.dao import BaseMenuDAO
from gwlib.services import BaseUserService


class BaseMenuService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseMenuDAO()

    def get_user_menu(self, user_id):
        user_service = BaseUserService()
        return user_service.get_menu(user_id=user_id)



from flask_login import UserMixin
from sqlalchemy import func
from sqlalchemy.orm import relationship

from gwlib.base.errors import TypeUserNotDefined, PolicyRoleInvalid
from gwlib.utils.helper import Helper
from . import db, user_golf_course
from .model_json import ModelJson


class User(db.Model, ModelJson, UserMixin):
    __tablename__ = "gw_user"

    user_id = db.Column(db.Integer, primary_key=True)
    type_id = db.Column(db.Integer, db.ForeignKey('gw_user_type.type_id'), nullable=False)
    name = db.Column(db.String(length=50), nullable=False)
    last_name = db.Column(db.String(length=50), nullable=False)
    username = db.Column(db.String(length=100), nullable=False, unique=True)
    email = db.Column(db.String(length=100), nullable=True, unique=True)
    password = db.Column(db.String(length=100), nullable=False)
    phone = db.Column(db.String(length=13), nullable=False)
    status = db.Column(db.String(length=10), nullable=False)
    photo_url = db.Column(db.String(length=250))
    deleted = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)

    golf_courses = db.relationship(
        "GolfCourse",
        secondary=user_golf_course,
        passive_deletes=True,
        backref="golf_course_user")

    user_type = relationship("UserType", back_populates="users")

    def validate_password(self, password):
        return Helper.verify_crypt(password, self.password)

    def get_id(self):
        return self.user_id

    def set_password(self, passowrd):
        self.password = Helper.set_crypt(passowrd)

    def get_permission_by_resource(self, resource):
        if not self.user_type:
            raise TypeUserNotDefined(self.email)

        resource_policies = self.user_type.user_types_policies.filter_by(
            resource_id=resource.resource_id).one_or_none()

        if not resource_policies:
            raise PolicyRoleInvalid(self.email)

        return resource_policies.to_json()



from gwlib.base.base_service import BaseService
from gwlib.dao.golf_course_dao import BaseGolfCourseDAO
from gwlib.models import GolfCourse


class BaseGolfCourseService(BaseService):
    required_fields = ["email"]

    def __init__(self):
        super().__init__()
        self.dao = BaseGolfCourseDAO()

    def get_obj_by_id(self, golf_course_id):
        filters = {
            "course_id": golf_course_id
        }
        return self.dao.get(**filters)

    def get_users(self, course_id):
        golf_course = self.get_obj_by_id(course_id)  # type: GolfCourse
        return golf_course.users
